package payroll;

import java.util.Set;
import java.util.HashSet;

import org.springframework.boot.Banner;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.WebApplicationType;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

@SpringBootApplication
public class ConsumingRestApplication {

	public static void main(String[] args) {
		System.out.println("Starting ConsumingRestApplication, #args = " + args.length);
		String appCase = null;  // web or SystemTest or ...
		SpringApplication app = new SpringApplication(ConsumingRestApplication.class);

		if (args.length == 0) {
			appCase = "client-server";
		} else appCase = args[0];
		if (!appCase.equals("web")) {
			System.out.println("assuming client execution");
			app.setBannerMode(Banner.Mode.OFF);
			app.setWebApplicationType(WebApplicationType.NONE); // don't start tomcat
		}
		app.run(args);  // get tomcat running if appropriate, and its DispatcherServlet, 
	}

	@Bean
	public RestTemplate restTemplate(RestTemplateBuilder builder) {
		return builder.build();
	}

	@Bean
	public CommandLineRunner run(RestTemplate restTemplate) throws Exception {
		return args -> {
			String url = "http://localhost:8083/employees";
			@SuppressWarnings(value = { "unchecked" })
			Set<Employee> emps = restTemplate.getForObject(url, HashSet.class);
			System.out.println("did GET to /employees, got: "+ emps);
	
			Employee newEmp = new Employee("newguy", "developer");
			HttpEntity<Employee> request = 
				      new HttpEntity<Employee>(newEmp, null);			
			Employee e = restTemplate.postForObject(url, request, Employee.class);
			System.out.println("Did POST of new Employee, got back: " + e);
			
			// get more info on response:
			ResponseEntity<Employee> e1 = restTemplate.postForEntity(url, request, Employee.class);
			System.out.println("Did POST of new Employee, got back: " + e1.getHeaders());
			
			@SuppressWarnings(value = { "unchecked" })
			Set<Employee> emps2 = restTemplate.getForObject(url, HashSet.class);
			System.out.println("did second GET to /employees, got: "+ emps2);	
		};
	}
}
